import * as THREE from 'three';
import { PerspectiveCamera, Mesh, WebGLRenderer, Object3D } from 'three';

declare type T = typeof THREE;
declare type K = keyof WebGLRenderer;
interface AnimateOptions {
    camera: PerspectiveCamera;
    elapsedTime: number;
    timestamp: number;
}
interface MiddlewareOptions {
    camera: PerspectiveCamera;
}
interface FnNameMap {
    cc: 'CubeCamera';
    v3: 'Vector3';
    v2: 'Vector2';
    v4: 'Vector4';
    oc: 'OrthographicCamera';
    pc: 'PerspectiveCamera';
    sc: 'StereoCamera';
    bg: 'BoxGeometry';
    ba: 'BufferAttribute';
    bufferg: 'BufferGeometry';
    capsuleg: 'CapsuleGeometry';
    coneg: 'ConeGeometry';
    cylinderg: 'CylinderGeometry';
    dg: 'DodecahedronGeometry';
    edgeg: 'EdgesGeometry';
    extrudeg: 'ExtrudeGeometry';
    ig: 'IcosahedronGeometry';
    lg: 'LatheGeometry';
    og: 'OctahedronGeometry';
    pg: 'PlaneGeometry';
    polyg: 'PolyhedronGeometry';
    rg: 'RingGeometry';
    sg: 'SphereGeometry';
    tetrag: 'TetrahedronGeometry';
    tg: 'TorusGeometry';
    tkg: 'TorusKnotGeometry';
    tubeg: 'TubeGeometry';
    wfg: 'WireframeGeometry';
    cg: 'CircleGeometry';
    ac: 'ArcCurve';
    crc3: 'CatmullRomCurve3';
    cbc: 'CubicBezierCurve';
    cbc3: 'CubicBezierCurve3';
    ec: 'EllipseCurve';
    lc: 'LineCurve';
    lc3: 'LineCurve3';
    qbc: 'QuadraticBezierCurve';
    qbc3: 'QuadraticBezierCurve3';
    splinec: 'SplineCurve';
    arrowh: 'ArrowHelper';
    ah: 'AxesHelper';
    bh: 'BoxHelper';
    b3h: 'Box3Helper';
    ch: 'CameraHelper';
    dlh: 'DirectionalLightHelper';
    gh: 'GridHelper';
    pgh: 'PolarGridHelper';
    hlh: 'HemisphereLightHelper';
    ph: 'PlaneHelper';
    plh: 'PointLightHelper';
    sh: 'SkeletonHelper';
    slh: 'SpotLightHelper';
    animationl: 'AnimationLoader';
    audiol: 'AudioLoader';
    bgl: 'BufferGeometryLoader';
    cache: 'Cache';
    c: 'Color';
    compressedtl: 'CompressedTextureLoader';
    ctl: 'CubeTextureLoader';
    dtl: 'DataTextureLoader';
    filel: 'FileLoader';
    fl: 'FontLoader';
    svgl: 'SVGLoader';
    il: 'ImageLoader';
    ibl: 'ImageBitmapLoader';
    l: 'Loader';
    lu: 'LoaderUtils';
    ml: 'MaterialLoader';
    ol: 'ObjectLoader';
    tl: 'TextureLoader';
    g: 'Group';
    lbm: 'LineBasicMaterial';
    ldm: 'LineDashedMaterial';
    m: 'Mesh';
    mbm: 'MeshBasicMaterial';
    mdepthm: 'MeshDepthMaterial';
    mdistancem: 'MeshDistanceMaterial';
    mlm: 'MeshLambertMaterial';
    mmm: 'MeshMatcapMaterial';
    mnm: 'MeshNormalMaterial';
    mpm: 'MeshPhongMaterial';
    mphysicalm: 'MeshPhysicalMaterial';
    msm: 'MeshStandardMaterial';
    mtm: 'MeshToonMaterial';
    p: 'Points';
    pm: 'PointsMaterial';
    rsm: 'RawShaderMaterial';
    sm: 'ShaderMaterial';
    shadowm: 'ShadowMaterial';
    spritem: 'SpriteMaterial';
    line: 'Line';
    lp: 'LineLoop';
    ls: 'LineSegments';
    al: 'AmbientLight';
    alp: 'AmbientLightProbe';
    dl: 'DirectionalLight';
    hl: 'HemisphereLight';
    hlp: 'HemisphereLightProbe';
    pl: 'PointLight';
    ral: 'RectAreaLight';
    sl: 'SpotLight';
    pls: 'PointLightShadow';
    dls: 'DirectionalLightShadow';
    sls: 'SpotLightShadow';
    lph: 'LightProbeHelper';
    ralh: 'RectAreaLightHelper';
    f: 'Fog';
    aa: 'AnimationAction';
    anc: 'AnimationClip';
    am: 'AnimationMixer';
    aog: 'AnimationObjectGroup';
    au: 'AnimationUtils';
    a: 'Animation';
    anl: 'AnimationLoader';
}
declare type ShadowType = 'BasicShadowMap' | 'PCFShadowMap' | 'PCFSoftShadowMap' | 'VSMShadowMap';
interface SThreeOptions extends Record<string, any> {
    createMesh: () => void;
    createCamera: () => PerspectiveCamera;
    animate?: (animationOptions: AnimateOptions) => void | PerspectiveCamera;
    middleware?: (middlewareOptions: MiddlewareOptions) => any;
    mousemove?: (e: Event) => void;
    mousedown?: (e: Event) => void;
    mouseup?: (e: Event) => void;
    debug?: boolean;
    alias?: Record<string, string>;
    shadowType?: ShadowType;
}
interface Scene extends Object3D {
    _add?: (...args: any[]) => void;
}
interface ReturnType {
    c: (fnName: keyof FnNameMap | keyof T, ...args: any[]) => any;
    setUV: (target: Mesh, size?: number) => void;
    animationArray: Mesh[];
    THREE: T;
    scene: Scene;
    renderer: WebGLRenderer;
    dom: HTMLCanvasElement;
    setRendererAttributes: (options: Record<K, any>) => void;
}
declare function sThree(container: HTMLElement | string, options: SThreeOptions): ReturnType;

export { sThree };
